<!doctype html>
<html lang="en">
<head>
  <meta charset="UTF-8" />
  <meta name="viewport" content="width=device-width, initial-scale=1.0">

  <title>{{ title }}</title>
  <link href="/www/css/bootstrap.min.css" rel="stylesheet" media="screen">
  <link href="/www/css/bootstrapSwitch.css" rel="stylesheet" media="screen">
  <link href="/www/css/bootstrap-responsive.css" rel="stylesheet">
</head>

<body>
<script src="/www/js/jquery-latest.js"></script>
<script src="/www/js/bootstrapSwitch.js"></script>
<script src="/www/js/bootstrap.min.js"></script>
<div class="container">
<table class="table table-hover table-condensed table-striped" id="myswitch">
    <tbody>
    {% for event in events %}
        <tr>
            <td >{{ events[event]['date'] }}</td>
            <td >{{ events[event]['tag'] }}</td>
            <td >{{ events[event]['value'] }}</td>
        </tr>
    {% endfor %}
    </tbody>
</table>
</div>

</body>
</html>
